/* Sony QRIO robot
 * by Marco Zaratti - marco.zaratti@gmail.com
*/
class QRIO extends KRobot config(USARBot);

var bool defaultPose;
var int loAngleLimit[22];
var int hiAngleLimit[22];
var int CameraFOV;
//Stereo camera vars
var config bool isStereo;
var config vector stereoSpacing; //(eye to eye)/2 spacing in UU
var vector uuStereoSpacing;

simulated function ConvertParam(USARConverter converter)
{
	Super.ConvertParam(converter);

	if (converter!=None) {
		uuStereoSpacing = converter.LengthVectorToUU(stereoSpacing);
	} else {
		uuStereoSpacing = stereoSpacing;
	}
}

function ProcessCarInput()
{
	local int nJoints;

	Super.ProcessCarInput();

	//Set the starting pose
	if(!defaultPose)
	{
		//Set here default joint positions.
		//RLE
		JointsControl[14].state = 1;
		JointsControl[14].order = 0;
		JointsControl[14].value = 1000;
		//LLE
		JointsControl[20].state = 1;
		JointsControl[20].order = 0;
		JointsControl[20].value = 1000;
		//BA
//		JointsControl[8].state = 1;
//		JointsControl[8].order = 0;
//		JointsControl[8].value = 910;
		bNewCommand = true;
		USARRemoteBot(Controller).myCameraZoom = CameraFOV;
		defaultPose=true;
	}

	//Limit joints order 0 angles
	for(nJoints=0; nJoints < JointsControl.length; nJoints++)
		if((JointsControl[nJoints].state == 1) && (JointsControl[nJoints].order == 0))
		{
			if(JointsControl[nJoints].value < loAngleLimit[nJoints])
			{
				JointsControl[nJoints].value = loAngleLimit[nJoints];
				continue;
			}
			if(JointsControl[nJoints].value > hiAngleLimit[nJoints])
				JointsControl[nJoints].value = hiAngleLimit[nJoints];
		}
}

//Stereo vision when viewing from local robot camera
simulated function DrawHud(Canvas C)
{
	local int halfSizeY;
	local vector stereoRotated;

    if(isStereo)
    {
    	halfSizeY = C.SizeY/2.0;
    	stereoRotated =  uuStereoSpacing >> myCamera.Rotation;

    	C.Reset();
    	C.Clear();	//Clears frame & Z buffers

    	//left Eye:
    	C.DrawPortal(0, 0, C.SizeX, halfSizeY, myCamera, myCamera.Location - stereoRotated, myCamera.Rotation, CameraZoom);

    	//Right Eye:
    	C.DrawPortal(0, halfSizeY, C.SizeX, halfSizeY, myCamera, myCamera.Location + stereoRotated, myCamera.Rotation, CameraZoom);
    }
	super.DrawHud(C);
}

//(Called by KRobot Tick function)
//Stereo vision when viewing from multiview robot camera
simulated function SyncMultiView()
{
	if(ViewManager != none)
	{
	   if(isStereo)
    	   ViewManager.UpdateView(ViewNum, myCamera.Location, myCamera.Rotation, CameraZoom, true, uuStereoSpacing);
    	else
    	   ViewManager.UpdateView(ViewNum, myCamera.Location, myCamera.Rotation, CameraZoom, false);
	}
}

defaultproperties
{
    DrawScale=0.32
	bVehicleShadows=false
	bDrawVehicleShadow=false
	bDebug=false
	CameraFOV=60
	isStereo=true
    stereoSpacing=(Y=0.05) //5 cm: (eye to eye)/2 spacing in meter

	//Low Joint Angle Limits
	loAngleLimit(0)=-5214     //HT
    loAngleLimit(1)=-20753     //HP
	loAngleLimit(2)=-31311     //RAA
	loAngleLimit(3)=0          //RAB
	loAngleLimit(4)=0          //RAC
	loAngleLimit(5)=-31311     //LAA
	loAngleLimit(6)=0          //LAB
	loAngleLimit(7)=0          //LAC
	loAngleLimit(8)=-10376     //BA
	loAngleLimit(9)=-5097      //BB
	loAngleLimit(10)=-14563     //RLA
	loAngleLimit(11)=-9284     //RLB
	loAngleLimit(12)=-5097     //RLC
	loAngleLimit(13)=-2002     //RLD
	loAngleLimit(14)=-7281     //RLE
	loAngleLimit(15)=-7281     //RLF
	loAngleLimit(16)=-14563    //LLA
	loAngleLimit(17)=-9284     //LLB
	loAngleLimit(18)=-5097     //LLC
	loAngleLimit(19)=-2002     //LLD
	loAngleLimit(20)=-7281     //LLE
	loAngleLimit(21)=-7281     //LLF
	//High Joint Angle Limits
	hiAngleLimit(0)=8342      //HT
    hiAngleLimit(1)=20753      //HP
	hiAngleLimit(2)=31311      //RAA
	hiAngleLimit(3)=32768      //RAB
	hiAngleLimit(4)=29127      //RAC
	hiAngleLimit(5)=31311      //LAA
	hiAngleLimit(6)=32768      //LAB
	hiAngleLimit(7)=29127      //LAC
	hiAngleLimit(8)=20753      //BA
	hiAngleLimit(9)=5097       //BB
	hiAngleLimit(10)=17658      //RLA
	hiAngleLimit(11)=16566     //RLB
	hiAngleLimit(12)=10376     //RLC
	hiAngleLimit(13)=26032     //RLD
	hiAngleLimit(14)=14563     //RLE
	hiAngleLimit(15)=7281      //RLF
	hiAngleLimit(16)=17658     //LLA
	hiAngleLimit(17)=16566     //LLB
	hiAngleLimit(18)=10376     //LLC
	hiAngleLimit(19)=26032     //LLD
	hiAngleLimit(20)=14563     //LLE
	hiAngleLimit(21)=7281      //LLF

	StaticMesh=StaticMesh'spqrRobotMeshes.QRIO.body'

    //Parameters of chassis. (KMass is defined in USAR.INI)
    Begin Object Class=KarmaParams Name=KParams0
        KStartEnabled=True
        bKDoubleTickRate=True
        bHighDetailOnly=False
        bClientOnly=False
        bKNonSphericalInertia=true
//Uncomment and define these when using RBfull karma params
//        KInertiaTensor(0)=0.4
//        KInertiaTensor(1)=0
//        KInertiaTensor(2)=0
//        KInertiaTensor(3)=0.4
//        KInertiaTensor(4)=0
//        KInertiaTensor(5)=0.4
//        KCOMOffset=(X=0.0,Y=0.0,Z=0.0)
        KMaxAngularSpeed=100
        KMaxSpeed=25000
        KLinearDamping=0.000000
        KAngularDamping=0.000000
        KFriction=0.8
        KRestitution=0.0
        Name="KParams0"
    End Object
    KParams=KarmaParams'KParams0'

//QRIO scale 250:
/*
[USARBot.QRIO]
msgTimer=0.2
ChassisMass=1.0
MaxTorque=60.0
MotorTorque=50.0
MotorSpeed=1.92
HingePropGap=1820
bDesiredBehindView=False
MaxNetUpdateInterval=0.1
bMountByUU=True
JointParts=(PartName="HT",PartClass=class'USARBot.QRIODummy',DrawScale3D=(X=0.2,Y=0.2,Z=0.2),BrakeTorque=50,Parent="",JointClass=class'USARBot.KDHinge',ParentPos=(X=8.025,Y=0.425,Z=61),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="HP",PartClass=class'USARBot.QRIOHead',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="HT",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(Z=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(Z=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="RAA",PartClass=class'USARBot.QRIODummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="",JointClass=class'USARBot.KDHinge',ParentPos=(X=8.026,Y=52.147,Z=25.997),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="RAB",PartClass=class'USARBot.QRIORAB',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RAA",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=1.0),ParentAxis2=(Y=1.0),SelfAxis=(X=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="RAC",PartClass=class'USARBot.QRIORAC',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RAB",JointClass=class'USARBot.KDHinge',ParentPos=(X=5.911,Y=-0.002,Z=-47.306),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="LAA",PartClass=class'USARBot.QRIODummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="",JointClass=class'USARBot.KDHinge',ParentPos=(X=8.026,Y=-51.305,Z=25.999),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="LAB",PartClass=class'USARBot.QRIOLAB',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LAA",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(X=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="LAC",PartClass=class'USARBot.QRIOLAC',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LAB",JointClass=class'USARBot.KDHinge',ParentPos=(X=5.911,Y=-0.005,Z=-47.304),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="BA",PartClass=class'USARBot.QRIODummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="",JointClass=class'USARBot.KDHinge',ParentPos=(X=8.026,Y=0.425,Z=-44.45),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="BB",PartClass=class'USARBot.QRIOBack',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="BA",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=1.0),ParentAxis2=(Y=1.0),SelfAxis=(X=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="RLA",PartClass=class'USARBot.QRIODummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="BB",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=23.64,Z=-27.587),ParentAxis=(Z=1.0),ParentAxis2=(Y=1.0),SelfAxis=(Z=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="RLB",PartClass=class'USARBot.QRIODummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RLA",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="RLC",PartClass=class'USARBot.QRIOLEC',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RLB",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=1.0),ParentAxis2=(Y=1.0),SelfAxis=(X=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="RLD",PartClass=class'USARBot.QRIOLED',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RLC",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=-57.157),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="RLE",PartClass=class'USARBot.QRIODummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RLD",JointClass=class'USARBot.KDHinge',ParentPos=(X=2.943,Y=0.0,Z=-51.217),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="RLF",PartClass=class'USARBot.QRIOLEF',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RLE",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=1.0),ParentAxis2=(Y=1.0),SelfAxis=(X=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="LLA",PartClass=class'USARBot.QRIODummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="BB",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=-23.64,Z=-27.587),ParentAxis=(Z=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(Z=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="LLB",PartClass=class'USARBot.QRIODummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LLA",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="LLC",PartClass=class'USARBot.QRIOLEC',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LLB",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(X=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="LLD",PartClass=class'USARBot.QRIOLED',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LLC",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=-57.157),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="LLE",PartClass=class'USARBot.QRIODummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LLD",JointClass=class'USARBot.KDHinge',ParentPos=(X=2.943,Y=0.0,Z=-51.217),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="LLF",PartClass=class'USARBot.QRIOLEF',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LLE",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(X=-1.0),SelfAxis2=(Y=-1.0))
Sensors=(ItemClass=Class'USARBot.BallHSensor',Parent="HP",ItemName="BHS",Position=(X=25.0,Y=0.0,Z=55.0),direction=(Y=0,Z=0,X=0))
Cameras=(ItemClass=Class'USARBot.AIBOCamera',Parent="HP",ItemName="Camera",Position=(X=25.0,Y=0.0,Z=55.0),direction=(Y=0,Z=0,X=0))
*/
//QRIO scale 80:
/*
JointParts=(PartName="HT",PartClass=class'USARBot.QRIODummy',DrawScale3D=(X=0.2,Y=0.2,Z=0.2),BrakeTorque=50,Parent="",JointClass=class'USARBot.KDHinge',ParentPos=(X=2.568,Y=0.136,Z=19.52),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="HP",PartClass=class'USARBot.QRIOHead',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="HT",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(Z=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(Z=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="RAA",PartClass=class'USARBot.QRIODummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="",JointClass=class'USARBot.KDHinge',ParentPos=(X=2.568,Y=16.687,Z=8.31904),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="RAB",PartClass=class'USARBot.QRIORAB',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RAA",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=1.0),ParentAxis2=(Y=1.0),SelfAxis=(X=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="RAC",PartClass=class'USARBot.QRIORAC',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RAB",JointClass=class'USARBot.KDHinge',ParentPos=(X=1.892,Y=0.001,Z=-15.138),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="LAA",PartClass=class'USARBot.QRIODummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="",JointClass=class'USARBot.KDHinge',ParentPos=(X=2.568,Y=-16.418,Z=8.32),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="LAB",PartClass=class'USARBot.QRIOLAB',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LAA",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(X=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="LAC",PartClass=class'USARBot.QRIOLAC',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LAB",JointClass=class'USARBot.KDHinge',ParentPos=(X=1.892,Y=-0.002,Z=-15.138),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="BA",PartClass=class'USARBot.QRIODummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="",JointClass=class'USARBot.KDHinge',ParentPos=(X=2.568,Y=0.136,Z=-14.224),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="BB",PartClass=class'USARBot.QRIOBack',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="BA",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=1.0),ParentAxis2=(Y=1.0),SelfAxis=(X=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="RLA",PartClass=class'USARBot.QRIODummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="BB",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=7.565,Z=-8.828),ParentAxis=(Z=1.0),ParentAxis2=(Y=1.0),SelfAxis=(Z=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="RLB",PartClass=class'USARBot.QRIODummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RLA",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="RLC",PartClass=class'USARBot.QRIOLEC',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RLB",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=1.0),ParentAxis2=(Y=1.0),SelfAxis=(X=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="RLD",PartClass=class'USARBot.QRIOLED',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RLC",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=-18.29),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="RLE",PartClass=class'USARBot.QRIODummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RLD",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.942,Y=0.0,Z=-16.389),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="RLF",PartClass=class'USARBot.QRIOLEF',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="RLE",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=1.0),ParentAxis2=(Y=1.0),SelfAxis=(X=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="LLA",PartClass=class'USARBot.QRIODummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="BB",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=-7.565,Z=-8.828),ParentAxis=(Z=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(Z=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="LLB",PartClass=class'USARBot.QRIODummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LLA",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="LLC",PartClass=class'USARBot.QRIOLEC',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LLB",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(X=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="LLD",PartClass=class'USARBot.QRIOLED',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LLC",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=-18.29),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="LLE",PartClass=class'USARBot.QRIODummy',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LLD",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.942,Y=0.0,Z=-16.389),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="LLF",PartClass=class'USARBot.QRIOLEF',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),BrakeTorque=50,Parent="LLE",JointClass=class'USARBot.KDHinge',ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(X=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(X=-1.0),SelfAxis2=(Y=-1.0))
Sensors=(ItemClass=Class'USARBot.BallHSensor',Parent="HP",ItemName="BHS",Position=(X=8.0,Y=0.0,Z=17.6),direction=(Y=0,Z=0,X=0))
Cameras=(ItemClass=Class'USARBot.AIBOCamera',Parent="HP",ItemName="Camera",Position=(X=8.0,Y=0.0,Z=17.6),direction=(Y=0,Z=0,X=0))
*/
}
